@@ -4,52 +4,38 @@ Third-party application inclusion support.
 
 from email.utils import parsedate
 from time import mktime
+from random import SystemRandom
 import re
 
-from django.core import urlresolvers
+from django.conf import settings
+from django.core.cache import cache
 from django.core.urlresolvers import Resolver404, resolve, reverse, NoReverseMatch
 from django.db import models
 from django.db.models import signals
 from django.http import HttpResponse
 from django.utils.functional import curry as partial, lazy, wraps
 from django.utils.safestring import mark_safe
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import get_language, ugettext_lazy as _
 
-from feincms import settings
 from feincms.admin.item_editor import ItemEditorForm
 from feincms.contrib.fields import JSONField
+from feincms.translations import short_language_code
 from feincms.utils import get_object
 
-try:
-    from threading import local
-except ImportError:
-    from django.utils._threading_local import local
 
-# Used to store MPTT informations about the currently requested
-# page. The information will be used to find the best application
-# content instance if a particular application has been added
-# more than once to the current website.
-# Additionally, we store the page class too, because when we have
-# more than one page class, reverse() will want to prefer the page
-# class used to render the current page. (See issue #240)
-_local = local()
+def cycle_app_reverse_cache(*args, **kwargs):
+    """Does not really empty the cache; instead it adds a random element to the
+    cache key generation which guarantees that the cache does not yet contain
+    values for all newly generated keys"""
+    cache.set('app_reverse_cache_generation', str(SystemRandom().random()))
 
 
-def retrieve_page_information(page, request=None):
-    """This is the request processor responsible for retrieving information
-    about the currently processed page so that we can make an optimal match
-    when reversing app URLs when the same ApplicationContent has been added
-    several times to the website."""
-    _local.proximity_info = (page.tree_id, page.lft, page.rght, page.level)
-    _local.page_class = page.__class__
-    _local.page_cache_key_fn = page.cache_key
+# Set the app_reverse_cache_generation value once per startup (at least).
+# This protects us against offline modifications of the database.
+cycle_app_reverse_cache()
 
 
-def _empty_reverse_cache(*args, **kwargs):
-    _local.reverse_cache = {}
-
-
-def app_reverse(viewname, urlconf, args=None, kwargs=None, prefix=None,
+def app_reverse(viewname, urlconf=None, args=None, kwargs=None, prefix=None,
         *vargs, **vkwargs):
     """
     Reverse URLs from application contents
@@ -75,108 +61,46 @@ def app_reverse(viewname, urlconf, args=None, kwargs=None, prefix=None,
     appconfig = extra_context.get('app_config', {})
     urlconf = appconfig.get('urlconf_path', urlconf)
 
-    # vargs and vkwargs are used to send through additional parameters which
-    # are uninteresting to us (such as current_app)
+    cache_generation = cache.get('app_reverse_cache_generation')
+    if cache_generation is None:
+        # This might never happen. Still, better be safe than sorry.
+        cycle_app_reverse_cache()
+        cache_generation = cache.get('app_reverse_cache_generation')
 
-    # get additional cache keys from the page if available
-    # refs https://github.com/feincms/feincms/pull/277/
-    fn = getattr(_local, 'page_cache_key_fn', lambda: '')
-    cache_key_prefix = fn()
+    cache_key = '%s-%s-%s-%s' % (
+        urlconf,
+        get_language(),
+        getattr(settings, 'SITE_ID', 0),
+        cache_generation)
 
-    app_cache_keys = {
-        'none': '%s:app_%s_none' % (cache_key_prefix, urlconf),
-        }
-    proximity_info = getattr(_local, 'proximity_info', None)
-    url_prefix = None
+    url_prefix = cache.get(cache_key)
 
-    if proximity_info:
-        app_cache_keys.update({
-            'all': '%s:app_%s_%s_%s_%s_%s' % (
-                (cache_key_prefix, urlconf,) + proximity_info),
-            'tree': '%s:app_%s_%s' % (
-                cache_key_prefix, urlconf, proximity_info[0]),
-            })
+    if url_prefix is None:
+        appcontent_class = ApplicationContent._feincms_content_models[0]
+        content = appcontent_class.closest_match(urlconf)
 
-    for key in ('all', 'tree', 'none'):
-        try:
-            url_prefix = _local.reverse_cache[app_cache_keys[key]]
-            break
-        except (AttributeError, KeyError):
-            pass
-    else:
-        try:
-            # Take the ApplicationContent class used by the current request
-            model_class = _local.page_class.content_type_for(ApplicationContent)
-        except AttributeError:
-            model_class = None
-
-        if not model_class:
-            # Take any
-            model_class = ApplicationContent._feincms_content_models[0]
-
-        # TODO: Only active pages? What about multisite support?
-        contents = model_class.objects.filter(
-            urlconf_path=urlconf).select_related('parent')
-
-        if proximity_info:
-            # find the closest match within the same subtree
-            tree_contents = contents.filter(parent__tree_id=proximity_info[0])
-            if not len(tree_contents):
-                # no application contents within the same tree
-                cache_key = 'tree'
-                try:
-                    content = contents[0]
-                except IndexError:
-                    content = None
-            elif len(tree_contents) == 1:
-                cache_key = 'tree'
-                # just one match within the tree, use it
-                content = tree_contents[0]
-            else: # len(tree_contents) > 1
-                cache_key = 'all'
-                try:
-                    # select all ancestors and descendants and get the one with
-                    # the smallest difference in levels
-                    content = (tree_contents.filter(
-                        parent__rght__gt=proximity_info[2],
-                        parent__lft__lt=proximity_info[1]
-                    ) | tree_contents.filter(
-                        parent__lft__lte=proximity_info[2],
-                        parent__lft__gte=proximity_info[1],
-                    )).extra({
-                        'level_diff': "abs(100 + level - %d)" % proximity_info[3]
-                    }).order_by('level_diff')[0]
-                except IndexError:
-                    content = tree_contents[0]
-        else:
-            cache_key = 'none'
-            try:
-                content = contents[0]
-            except IndexError:
-                content = None
-
-        if content:
-            if urlconf in model_class.ALL_APPS_CONFIG:
+        if content is not None:
+            if urlconf in appcontent_class.ALL_APPS_CONFIG:
                 # We have an overridden URLconf
-                urlconf = model_class.ALL_APPS_CONFIG[urlconf]['config'].get(
-                    'urls', urlconf)
-
-            if not hasattr(_local, 'reverse_cache'):
-                _local.reverse_cache = {}
+                app_config = appcontent_class.ALL_APPS_CONFIG[urlconf]
+                urlconf = app_config['config'].get('urls', urlconf)
 
             prefix = content.parent.get_absolute_url()
             prefix += '/' if prefix[-1] != '/' else ''
 
-            _local.reverse_cache[app_cache_keys[cache_key]] = url_prefix = (
-                urlconf, prefix)
+            url_prefix = (urlconf, prefix)
+            cache.set(cache_key, url_prefix)
 
     if url_prefix:
+        # vargs and vkwargs are used to send through additional parameters
+        # which are uninteresting to us (such as current_app)
         return reverse(viewname,
             url_prefix[0],
             args=args,
             kwargs=kwargs,
             prefix=url_prefix[1],
             *vargs, **vkwargs)
+
     raise NoReverseMatch("Unable to find ApplicationContent for %r" % urlconf)
 
 
@@ -240,9 +164,9 @@ class ApplicationContent(models.Model):
                 app_conf = {}
 
             cls.ALL_APPS_CONFIG[urls] = {
-                "urls":     urls,
-                "name":     name,
-                "config":   app_conf
+                "urls": urls,
+                "name": name,
+                "config": app_conf
             }
 
         cls.add_to_class('urlconf_path',
@@ -251,11 +175,12 @@ class ApplicationContent(models.Model):
         )
 
         class ApplicationContentItemEditorForm(ItemEditorForm):
-            app_config    = {}
+            app_config = {}
             custom_fields = {}
 
             def __init__(self, *args, **kwargs):
-                super(ApplicationContentItemEditorForm, self).__init__(*args, **kwargs)
+                super(ApplicationContentItemEditorForm, self).__init__(
+                    *args, **kwargs)
 
                 instance = kwargs.get("instance", None)
 
@@ -303,12 +228,13 @@ class ApplicationContent(models.Model):
         # embedded instances:
         cls.feincms_item_editor_form = ApplicationContentItemEditorForm
 
-        # Make sure the patched reverse() method has all information it needs
-        cls.parent.field.rel.to.register_request_processor(
-            retrieve_page_information)
-
-        signals.post_save.connect(_empty_reverse_cache, sender=cls)
-        signals.post_delete.connect(_empty_reverse_cache, sender=cls)
+        # Clobber the app_reverse cache when saving application contents
+        # and/or pages
+        page_class = cls.parent.field.rel.to
+        signals.post_save.connect(cycle_app_reverse_cache, sender=cls)
+        signals.post_delete.connect(cycle_app_reverse_cache, sender=cls)
+        signals.post_save.connect(cycle_app_reverse_cache, sender=page_class)
+        signals.post_delete.connect(cycle_app_reverse_cache, sender=page_class)
 
     def __init__(self, *args, **kwargs):
         super(ApplicationContent, self).__init__(*args, **kwargs)
@@ -386,7 +312,7 @@ class ApplicationContent(models.Model):
         else:
             self.rendered_result = mark_safe(output)
 
-        return True # successful
+        return True  # successful
 
     def send_directly(self, request, response):
         mimetype = response.get('Content-Type', 'text/plain')
@@ -435,3 +361,30 @@ class ApplicationContent(models.Model):
         lm_list = [parsedate(x) for x in headers.get('Expires', ())]
         if len(lm_list) > 0:
             response['Expires'] = http_date(mktime(min(lm_list)))
+
+    @classmethod
+    def closest_match(cls, urlconf_path):
+        page_class = cls.parent.field.rel.to
+
+        contents = cls.objects.filter(
+            parent__in=page_class.objects.active(),
+            urlconf_path=urlconf_path,
+            ).order_by('pk').select_related('parent')
+
+        if len(contents) > 1:
+            try:
+                current = short_language_code(get_language())
+                return [
+                    content for content in contents if
+                    short_language_code(content.parent.language) == current
+                    ][0]
+
+            except (AttributeError, IndexError):
+                pass
+
+        try:
+            return contents[0]
+        except IndexError:
+            pass
+
+        return None
