@@ -58,13 +58,14 @@ class RichTextContentAdminForm(ItemEditorForm):
 
         return cleaned_data
 
+
 class RichTextContent(models.Model):
     """
     Rich text content. Uses TinyMCE by default, but can be configured to do
     anything you want using ``FEINCMS_RICHTEXT_INIT_CONTEXT`` and
     ``FEINCMS_RICHTEXT_INIT_TEMPLATE``.
 
-    If you are using TinyMCE 4.x then ``FEINCMS_RICHTEXT_INIT_TEMPLATE`` 
+    If you are using TinyMCE 4.x then ``FEINCMS_RICHTEXT_INIT_TEMPLATE``
     needs to be set to ``admin/content/richtext/init_tinymce4.html``.
 
     Optionally runs the HTML code through HTML cleaners if you specify
@@ -78,7 +79,7 @@ class RichTextContent(models.Model):
         lambda x: settings.FEINCMS_RICHTEXT_INIT_CONTEXT,
     )
     feincms_item_editor_includes = {
-        'head': [ settings.FEINCMS_RICHTEXT_INIT_TEMPLATE ],
+        'head': [settings.FEINCMS_RICHTEXT_INIT_TEMPLATE],
     }
 
     text = RichTextField(_('text'), blank=True)
@@ -90,11 +91,11 @@ class RichTextContent(models.Model):
 
     def render(self, **kwargs):
         return render_to_string('content/richtext/default.html',
-            { 'content': self }, context_instance=kwargs.get('context'))
+            {'content': self}, context_instance=kwargs.get('context'))
 
     def save(self, *args, **kwargs):
         # TODO: Move this to the form?
-        if getattr(self, 'cleanse', False):
+        if getattr(self, 'cleanse', None):
             # Passes the rich text content as first argument because
             # the passed callable has been converted into a bound method
             self.text = self.cleanse(self.text)
@@ -103,35 +104,19 @@ class RichTextContent(models.Model):
     save.alters_data = True
 
     @classmethod
-    def initialize_type(cls, cleanse=False):
+    def initialize_type(cls, cleanse=None):
         def to_instance_method(func):
             def func_im(self, *args, **kwargs):
                 return func(*args, **kwargs)
             return func_im
 
         if cleanse:
-            # If cleanse is True use default cleanse method
-            if cleanse == True:
-                import warnings
-                warnings.warn("Please pass a callable instead. cleanse=True is"
-                    " being deprecated in favor of explicitly specifying the"
-                    " cleansing function. To continue using the same"
-                    " functionality, pip install feincms-cleanse and pass"
-                    " cleanse=feincms_cleanse.cleanse_html to the"
-                    " create_content_type call."
-                    " Support for cleanse=True will be removed in FeinCMS v1.8.",
-                    DeprecationWarning, stacklevel=2)
-
-                from feincms.utils.html.cleanse import cleanse_html
-                cls.cleanse = to_instance_method(cleanse_html)
-            # Otherwise use passed callable
-            else:
-                cls.cleanse = to_instance_method(cleanse)
+            cls.cleanse = to_instance_method(cleanse)
 
         # TODO: Move this into somewhere more generic:
         if settings.FEINCMS_TIDY_HTML:
             # Make sure we can load the tidy function without dependency failures:
             try:
                 get_object(settings.FEINCMS_TIDY_FUNCTION)
-            except ImportError, e:
+            except ImportError as e:
                 raise ImproperlyConfigured("FEINCMS_TIDY_HTML is enabled but the HTML tidy function %s could not be imported: %s" % (settings.FEINCMS_TIDY_FUNCTION, e))
