@@ -1,26 +1,26 @@
-try:
-    import json
-except ImportError:
-    from django.utils import simplejson as json  # Python 2.5
+import json
 import logging
 
 from django import forms
 from django.db import models
 from django.core.serializers.json import DjangoJSONEncoder
+from django.utils import six
 
 
 class JSONFormField(forms.fields.CharField):
     def clean(self, value, *args, **kwargs):
         if value:
             try:
-                # Run the value through JSON so we can normalize formatting and at least learn about malformed data:
+                # Run the value through JSON so we can normalize formatting
+                # and at least learn about malformed data:
                 value = json.dumps(json.loads(value), cls=DjangoJSONEncoder)
             except ValueError:
                 raise forms.ValidationError("Invalid JSON data!")
 
         return super(JSONFormField, self).clean(value, *args, **kwargs)
 
-class JSONField(models.TextField):
+
+class JSONField(six.with_metaclass(models.SubfieldBase, models.TextField)):
     """
     TextField which transparently serializes/unserializes JSON objects
 
@@ -28,9 +28,6 @@ class JSONField(models.TextField):
     http://www.djangosnippets.org/snippets/1478/
     """
 
-    # Used so to_python() is called
-    __metaclass__ = models.SubfieldBase
-
     formfield = JSONFormField
 
     def to_python(self, value):
@@ -38,7 +35,8 @@ class JSONField(models.TextField):
 
         if isinstance(value, dict):
             return value
-        elif isinstance(value, basestring):
+        elif (isinstance(value, six.string_types)
+                or isinstance(value, six.binary_type)):
             # Avoid asking the JSON decoder to handle empty values:
             if not value:
                 return {}
@@ -75,15 +73,18 @@ class JSONField(models.TextField):
         if isinstance(value, dict):
             value = json.dumps(value, cls=DjangoJSONEncoder)
 
-        assert isinstance(value, basestring)
+        assert isinstance(value, six.string_types)
 
         return value
 
+
 try:
     from south.modelsinspector import add_introspection_rules
 
-    JSONField_introspection_rule = ( (JSONField,), [], {}, )
+    JSONField_introspection_rule = ((JSONField,), [], {},)
 
-    add_introspection_rules(rules=[JSONField_introspection_rule], patterns=["^feincms\.contrib\.fields"])
+    add_introspection_rules(
+        rules=[JSONField_introspection_rule],
+        patterns=["^feincms\.contrib\.fields"])
 except ImportError:
     pass
