@@ -2,12 +2,12 @@
 Base types for extensions refactor
 """
 
-import re
 import warnings
 from functools import wraps
 
 from django.contrib import admin
 from django.core.exceptions import ImproperlyConfigured
+from django.utils import six
 
 from feincms.utils import get_object
 
@@ -38,51 +38,20 @@ class ExtensionsMixin(object):
             cls._extensions = []
             cls._extensions_seen = []
 
-        here = cls.__module__.split('.')[:-1]
-        search_paths = [
-            '.'.join(here + ['extensions']),
-            '.'.join(here[:-1] + ['extensions']),
-            'feincms.module.extensions',
-            ]
-
         for ext in extensions:
             if ext in cls._extensions:
                 continue
 
             extension = None
 
-            if isinstance(ext, basestring):
-                paths = [ext, '%s.register' % ext]
-                for path in search_paths:
-                    paths.extend([
-                        '%s.%s.register' % (path, ext),
-                        '%s.%s' % (path, ext),
-                        ])
-
-                for idx, path in enumerate(paths):
-                    try:
-                        extension = get_object(path)
-
-                        if idx >= 2:
-                            warnings.warn(
-                                'Using short names for extensions has been'
-                                ' deprecated and will be removed in'
-                                ' FeinCMS v1.8. Please provide the full'
-                                ' python path to the extension'
-                                ' %s instead (%s).' % (
-                                    ext,
-                                    re.sub(r'\.register$', '', path),
-                                    ),
-                                DeprecationWarning, stacklevel=2)
-
-                        break
-                    except (AttributeError, ImportError, ValueError):
-                        pass
-
-            if not extension:
-                raise ImproperlyConfigured(
-                    '%s is not a valid extension for %s' % (
-                        ext, cls.__name__))
+            if isinstance(ext, six.string_types):
+                try:
+                    extension = get_object(ext)
+                except (AttributeError, ImportError, ValueError):
+                    if not extension:
+                        raise ImproperlyConfigured(
+                            '%s is not a valid extension for %s' % (
+                                ext, cls.__name__))
 
             if hasattr(extension, 'Extension'):
                 extension = extension.Extension
@@ -105,7 +74,12 @@ class ExtensionsMixin(object):
             if hasattr(extension, 'handle_model'):
                 cls._extensions.append(extension(cls))
             else:
-                cls._extensions.append(LegacyExtension(cls, extension=extension))
+                warning.warn(
+                    '%r is a extension in legacy format.'
+                    ' Support for legacy extensions will be removed in'
+                    ' FeinCMS v1.9. Convert your extensions to'
+                    ' feincms.extensions.Extension now.' % extension,
+                    DeprecationWarning)
 
 
 class Extension(object):
