@@ -6,7 +6,6 @@ from __future__ import absolute_import
 
 import re
 
-from django import forms
 from django.contrib.admin.widgets import ForeignKeyRawIdWidget
 from django.contrib.sites.models import Site
 from django.db.models.loading import get_model
@@ -16,7 +15,6 @@ from django.utils.safestring import mark_safe
 from django.utils.translation import ugettext_lazy as _
 
 from feincms import ensure_completely_loaded
-from feincms.utils import shorten_string
 
 from mptt.forms import MPTTAdminForm
 
@@ -64,6 +62,7 @@ class PageAdminForm(MPTTAdminForm):
                 try:
                     page = self.page_manager.get(
                         pk=kwargs['initial']['parent'])
+
                     data = model_to_dict(page)
 
                     for field in self.page_manager.exclude_from_copy:
@@ -76,6 +75,8 @@ class PageAdminForm(MPTTAdminForm):
                             del data[field]
 
                     data.update(kwargs['initial'])
+                    if page.template.child_template:
+                        data['template_key'] = page.template.child_template
                     kwargs['initial'] = data
                 except self.page_model.DoesNotExist:
                     pass
@@ -117,17 +118,23 @@ class PageAdminForm(MPTTAdminForm):
                 self.page_model._meta.get_field('parent').rel,
                 modeladmin.admin_site)
 
-        if 'instance' in kwargs:
+        if 'template_key' in self.fields:
             choices = []
-            for key, template in kwargs['instance'].TEMPLATE_CHOICES:
-                template = kwargs['instance']._feincms_templates[key]
+            for key, template_name in self.page_model.TEMPLATE_CHOICES:
+                template = self.page_model._feincms_templates[key]
+                pages_for_template = self.page_model._default_manager.filter(
+                    template_key=key)
+                pk = kwargs['instance'].pk if 'instance' in kwargs else None
+                other_pages_for_template = pages_for_template.exclude(pk=pk)
+                if template.singleton and other_pages_for_template.exists():
+                    continue  # don't allow selection of singleton if in use
                 if template.preview_image:
                     choices.append((template.key,
                         mark_safe(u'<img src="%s" alt="%s" /> %s' % (
-                              template.preview_image,
-                              template.key,
-                              template.title,
-                              ))))
+                            template.preview_image,
+                            template.key,
+                            template.title,
+                            ))))
                 else:
                     choices.append((template.key, template.title))
 
@@ -190,6 +197,11 @@ class PageAdminForm(MPTTAdminForm):
             self._errors['active'] = ErrorList([_('This URL is already taken by another active page.')])
             del cleaned_data['active']
 
+        if parent and parent.template.enforce_leaf:
+            self._errors['parent'] = ErrorList(
+                [_('This page does not allow attachment of child pages')])
+            del cleaned_data['parent']
+
         return cleaned_data
 
 # ------------------------------------------------------------------------
