@@ -30,10 +30,16 @@ if(!Array.indexOf) {
     }
 
     function create_new_item_from_form(form, modname, modvar){
-        var fieldset = $("<fieldset>").addClass("module aligned order-item item-wrapper-" + modvar);
 
-        var wrp = [];
-        wrp.push('<h2><img class="item-delete" src="'+IMG_DELETELINK_PATH+'" /><span class="handle"></span> <span class="modname">'+modname+'</span> &nbsp;(<span class="collapse">'+feincms_gettext('Hide')+'</span>)</h2>');
+        var fieldset = $("<fieldset>").addClass("module aligned order-item item-wrapper-" + modvar);
+        var original_id_id = '#id_' + form.attr('id') + '-id';
+
+        var wrp = ['<h2>'];
+        // If original has delete checkbox or this is a freshly added CT? Add delete link!
+        if($('.delete', form).length || !$(original_id_id, form).val()) {
+            wrp.push('<img class="item-delete" src="'+IMG_DELETELINK_PATH+'" />');
+        }
+        wrp.push('<span class="handle"></span> <span class="modname">'+modname+'</span> &nbsp;(<span class="collapse">'+feincms_gettext('Hide')+'</span>)</h2>');
         wrp.push('<div class="item-content"></div>');
         fieldset.append(wrp.join(""));
 
@@ -105,17 +111,17 @@ if(!Array.indexOf) {
         var mouseenter_timeout;
         var mouseleave_timeout;
         function hide_controls() {
-            item_controls.find("*").fadeOut(800);
+            item_controls.find("*").fadeOut(400);
             is_hidden = true;
         }
         function show_controls() {
-            item_controls.find("*").fadeIn(800);
+            item_controls.find("*").fadeIn(200);
             is_hidden = false;
         }
         item_controls.unbind('mouseleave'); // Unbind in case it's already been bound.
         item_controls.mouseleave(function() {
             clearTimeout(mouseenter_timeout);
-            mouseleave_timeout = setTimeout(hide_controls, 1000);
+            mouseleave_timeout = setTimeout(hide_controls, 200);
         });
         item_controls.unbind('mouseenter'); // Unbind in case it's already been bound.
         item_controls.mouseenter(function() {
@@ -376,18 +382,22 @@ if(!Array.indexOf) {
                 if (r==true) {
                     var in_database = item.find(".delete-field").length;
                     if(in_database==0){ // remove on client-side only
-                        // decrement TOTAL_FORMS:
                         var id = item.find(".item-content > div").attr('id');
-                        var modvar = id.replace(/_set-\d+$/, '');
-                        var count = $('#id_'+modvar+'_set-TOTAL_FORMS').val();
-                        // remove form:
-                        item.find(".item-content").remove();
 
-                        // could trigger django's deletion handler, which would
-                        // handle reindexing other inlines, etc, but seems to
-                        // cause errors, and is apparently unnecessary...
-                        // django.jQuery('#'+id).find('a.inline-deletelink')
-                        //   .triggerHandler('click');
+                        // poorify all contents
+                        items = item.parents('.order-machine').find('.order-item');
+                        items.each(function() {
+                          poorify_rich($(this));
+                        })
+
+                        // remove content
+                        django.jQuery('#'+id).find('a.inline-deletelink')
+                          .triggerHandler('click');
+
+                        // richify all contents again
+                        items.each(function() {
+                          richify_poor($(this));
+                        })
                     }
                     else{ // saved on server, don't remove form
                         set_item_field_value(item,"delete-field","checked");
@@ -458,7 +468,7 @@ if(!Array.indexOf) {
                     /* Simulate a click on the save button instead of form.submit(), so
                        that the submit handlers from FilteredSelectMultiple get
                        invoked. See Issue #372 */
-                    $('#page_form input[type=submit][name=_save]').click();
+                    form_element.find('input[type=submit][name=_save]').click();
 
                 } else {
                     $("div#popup_bg").remove();
@@ -503,7 +513,7 @@ if(!Array.indexOf) {
                     elem.find(".region-choice-field").val());
                 if (REGION_MAP[region_id] != undefined) {
                     var content_type = elem.attr("id").substr(
-                        0, elem.attr("id").indexOf("_"));
+                        0, elem.attr("id").lastIndexOf("_"));
                     var item = create_new_item_from_form(
                         elem, CONTENT_NAMES[content_type], content_type);
                     add_fieldset(region_id, item, {where:'append'});
